package com.yaoyicloud.config;

import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

@Component
public class CommonDataCache {

    // relationId -> 通用数据映射
    private final ConcurrentMap<String, ConcurrentMap<String, Object>> dataCache
            = new ConcurrentHashMap<>();

    /**
     *
     */
    public void addData(String sessionId, Map<String, Object> data) {
        if (data != null) {
            // 使用putIfAbsent避免重复创建空Map
            dataCache.putIfAbsent(sessionId, new ConcurrentHashMap<>());
            // 直接替换整个数据Map
            dataCache.get(sessionId).putAll(data);
        }
    }

    /**
     * 添加或更新单个通用数据项
     */
    public void addDataItem(String sessionId, String key, Object value) {
        dataCache.computeIfAbsent(sessionId, k -> new ConcurrentHashMap<>())
                .put(key, value);
    }

    /**
     * 获取通用数据
     */
    public Map<String, Object> getData(String sessionId) {
        ConcurrentMap<String, Object> sessionData = dataCache.get(sessionId);
        return sessionData != null ? Collections.unmodifiableMap(sessionData) : Collections.emptyMap();
    }

    /**
     * 获取通用数据项
     */
    public Object getDataItem(String sessionId, String key) {
        ConcurrentMap<String, Object> sessionData = dataCache.get(sessionId);
        return sessionData != null ? sessionData.get(key) : null;
    }

    /**
     * 获取指定类型的通用数据项
     */
    @SuppressWarnings("unchecked")
    public <T> T getDataItem(String sessionId, String key, Class<T> type) {
        Object value = getDataItem(sessionId, key);
        return type.isInstance(value) ? (T) value : null;
    }

    /**
     * 移除指定session的通用数据
     */
    public Map<String, Object> removeSessionData(String sessionId) {
        ConcurrentMap<String, Object> removed = dataCache.remove(sessionId);
        return removed != null ? new HashMap<>(removed) : Collections.emptyMap();
    }

    /**
     * 移除指定session中的某个数据项
     */
    public Object removeDataItem(String sessionId, String key) {
        ConcurrentMap<String, Object> sessionData = dataCache.get(sessionId);
        return sessionData != null ? sessionData.remove(key) : null;
    }

    /**
     * 检查是否存在指定session的通用数据
     */
    public boolean hasSessionData(String sessionId) {
        return dataCache.containsKey(sessionId);
    }

    /**
     * 检查指定session中是否存在某个数据项
     */
    public boolean hasDataItem(String sessionId, String key) {
        ConcurrentMap<String, Object> sessionData = dataCache.get(sessionId);
        return sessionData != null && sessionData.containsKey(key);
    }

    /**
     * 获取所有sessionId
     */
    public java.util.Set<String> getAllSessionIds() {
        return dataCache.keySet();
    }

    /**
     * 清空所有缓存数据
     */
    public void clear() {
        dataCache.clear();
    }
}
