package com.yaoyicloud.config;

import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * 报告路径管理器
 */
@Component
public class ReportPathManager {

    // sessionId -> 报告路径队列（使用ConcurrentLinkedQueue保证有序且线程安全）
    private final ConcurrentMap<String, ConcurrentLinkedQueue<String>> sessionReportMap = new ConcurrentHashMap<>();

    /**
     * 添加单个报告路径（按插入顺序有序）
     */
    public void addReportPath(String sessionId, String reportPath) {
        // computeIfAbsent 确保原子性初始化
        sessionReportMap.computeIfAbsent(sessionId, k -> new ConcurrentLinkedQueue<>())
                .offer(reportPath);
    }

    /**
     * 批量添加报告路径（按集合迭代顺序插入，保证整体有序性）
     */
    public void addReportPaths(String sessionId, Collection<String> reportPaths) {
        ConcurrentLinkedQueue<String> queue = sessionReportMap.computeIfAbsent(sessionId, k -> new ConcurrentLinkedQueue<>());
        queue.addAll(reportPaths);
    }

    /**
     * 获取指定sessionId下的所有报告路径（按插入顺序返回）
     */
    public List<String> getReportPaths(String sessionId) {
        ConcurrentLinkedQueue<String> queue = sessionReportMap.get(sessionId);
        return queue != null ? new ArrayList<>(queue) : Collections.emptyList();
    }

    /**
     * 检查sessionId是否存在
     */
    public boolean hasSession(String sessionId) {
        return sessionReportMap.containsKey(sessionId);
    }

    /**
     * 获取所有sessionId
     */
    public Set<String> getAllSessions() {
        return sessionReportMap.keySet();
    }

    /**
     * 获取所有报告路径的总数
     */
    public int getTotalReportCount() {
        return sessionReportMap.values().stream()
                .mapToInt(ConcurrentLinkedQueue::size)
                .sum();
    }

    /**
     * 移除指定sessionId及其所有报告路径
     */
    public List<String> removeSession(String sessionId) {
        ConcurrentLinkedQueue<String> removedQueue = sessionReportMap.remove(sessionId);
        return removedQueue != null ? new ArrayList<>(removedQueue) : Collections.emptyList();
    }

    /**
     * 清空所有数据
     */
    public void clear() {
        sessionReportMap.clear();
    }
}